#ifndef __dosallocmem_h__
#define __dosallocmem_h__

#ifdef __WITH_FORK

#define INCL_DOSMEMMGR
#include <os2.h>

#if defined (__cplusplus)
extern "C" {
#endif

#define OBJ_FORK  0x10000000      /* allocate memory on given in ppb value */

typedef struct __libc_AllocMemLog
{
	void * pMemAddress;
	unsigned long ulMemSize;
} __LIBC_ALLOCMEMLOG, *__LIBC_PALLOCMEMLOG;

extern unsigned long __libc_gulAllocMemCnt;
extern __LIBC_PALLOCMEMLOG __libc_gpAllocMemLog;

/**
 * Allocating memory with DosAllocMem and log allocated address in pAllocMemLog array
 *
 * @returns	see DosAllocMem description.
 * @params	see DosAllocMem description.
 *
 * @remarks  	If OBJ_FORK flag is given, DosAllocMemEx will try to allocate  
 *		memory on address recorded in *ppb, this allocation will not be logged.
 */
ULONG APIENTRY DosAllocMemEx(PPVOID ppb, ULONG cb, ULONG flag);

/**
 * Freeing memory and delete from log of allocated address in pAllocMemLog array
 *
 * @returns	see DosFreeMem description.
 * @params	see DosFreeMem description.
 *
 */
ULONG APIENTRY DosFreeMemEx(PVOID pb);

#define __LIBC_MEMEX_NOFREEMEM	0x00000001
#define __LIBC_MEMEX_NOFREESEM	0x00000002

/**
 * Freeing memory from allocation log and free the log conrtainer
 *
 * @params	flags. Can be :
 *              __LIBC_MEMEX_NOFREEMEM - do not free memory from
 *		allocation log, free only log container
 *              __LIBC_MEMEX_NOFREESEM - do not close mutex sem used
 *		to protect allocation log
 *
 */
void __libc_clearMemEx(int iFlags);

/**
 * Sort memory log. Called prior to copy memory to child process.
 *
 */
void __libc_sortMemEx(void);


#define DosAllocMem(a, b, c) DosAllocMemEx((a),(b),(c))
#define DosFreeMem(a) DosFreeMemEx((a))

#if defined (__cplusplus)
}
#endif

#endif /* __WITH_FORK */

#endif /* __dosallocmem_h__ */
