/** @file
 *
 * LIBC Fork Handling.
 *
 *
 *
 * This file is part of InnoTek LIBC.
 *
 * InnoTek LIBC is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * InnoTek LIBC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with InnoTek LIBC; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifndef __InnoTekLIBC_fork_h__
#define __InnoTekLIBC_fork_h__

#ifdef __WITH_FORK

#if defined (__cplusplus)
extern "C" {
#endif

#pragma pack(1)

/* Size of fork buffer */
#define FORK_BUFFER_SIZE (1 << 20)
#define FORK_SEM_TIMEOUT (30*1000)

typedef enum __LIBC_FORKSIDE
{
	__LIBC_FORKSIDE_PARENT = 0,
	__LIBC_FORKSIDE_CHILD = 1,
	__LIBC_FORKSIDE_BOTH = 2,
} __LIBC_FORKSIDE;

typedef enum __LIBC_CALLBACKOPERATION
{
	__LIBC_FORK_CHECK_PARENT = 0,
	__LIBC_FORK_CHECK_CHILD = 1,
	__LIBC_FORK_FORK_PARENT = 2,
	__LIBC_FORK_FORK_CHILD = 3,
	__LIBC_FORK_COMPLETE_PARENT = 4,
	__LIBC_FORK_COMPLETE_CHILD = 5,
} __LIBC_CALLBACKOPERATION;

/* Copy only dirty pages */
#define __LIBC_FORKFLAG_ONLY_DIRTY	0x00000001
/* Allocate memory prior to copy it */
#define __LIBC_FORKFLAG_ALLOCATE	0x00000002
/* Copy memory with shared flag (libc has it in its private data segment) */
#define __LIBC_FORKFLAG_COPY_SHARED	0x00000004

#define __LIBC_FORK_INITIALIZED	1
#define __LIBC_FORK_FAILED	2

typedef struct __libc_ForkHandle
{
	unsigned long pForkModuleListHead;
	unsigned long pForkModuleListTail;
	int iFlags;
	/* owner of the fork handle */
	enum __LIBC_FORKSIDE enmSide;
	/* Pointer to the fork buffer */
	void * pSharedMem;
	/* Size of fork buffer */
	unsigned long ulSharedMemSize;
	unsigned long hmtxMutex;
	unsigned long hevEventParent;
	unsigned long hevEventChild;
	unsigned long ulSemTimeout;
	/* Logical start of stack (the highest address) */
	void * pStackStart;
	/* Logical end of stack (the lowest address). Not really used */
	void * pStackEnd;
	/* Current stack position. */
	void * pStackCurrent;
	/* Here the originate child's stack is kept. If we do fork not from the main thread, 
           it can be different then the stack from parent */
	/* Logical start of stack in child */
	void * pStackChildStart;
	/* Logical end of stack in child (the lowest address) */
	void * pStackChildEnd;

	/* 
	 * Pointer to the head of list of either parent callbacks (for parent forkmodule) 
	 * or child callbacks (for child forkmodule) sorted according their priority.
	 */
	void * pForkCallbackListHead;
	/* Fork primitives */
	int (*pfnDuplicatePages)(struct __libc_ForkHandle * pForkHandle, void * pMemStart, void * pMemEnd, int iFlags);
	int (*pfnFlush)(struct __libc_ForkHandle * pForkHandle);
	int (*pfnRegisterCompletionCallback)(struct __libc_ForkHandle * pForkHandle, void * pfnCallback, void *pvArg, enum __LIBC_FORKSIDE enmContext);
	int (*pfnCallCompletionCallback)(struct __libc_ForkHandle * pForkHandle, int iRc);
	int (*pfnMemDupToDest)(struct __libc_ForkHandle * pForkHandle, char * pMemDestination, char * pMemSource, unsigned long ulMemSize, unsigned long ulDataPosition, unsigned long ulDataSize, int iFlags);
} __LIBC_FORKHANDLE, *__LIBC_PFORKHANDLE;

/* The maximum size of __LIBC_FORKHANDLE (including future enhancements) This should be enough for everyone (c) Bill Gates */
#define MAX_FORKHANDLE_SIZE 1024
/* Size of shared memory allocated for fork handle plus registered fork modules */
#define FORKHANDLE_MEMSIZE 8192

typedef	int (*__LIBC_FORKCALLBACKFUNCTION)(void *pvForkModule,
		__LIBC_PFORKHANDLE pForkHandle, enum __LIBC_CALLBACKOPERATION enmOperation,
		void * pvForkBuffer);

typedef struct __libc_ForkCallback
{
	/* Initiator function, which fills pForkBuffer */
	__LIBC_FORKCALLBACKFUNCTION pfnCallbackInitiator;
	/* Processor function, which process content of pForkBuffer at other side. Used only for parent size callbacks */
	__LIBC_FORKCALLBACKFUNCTION pfnCallbackProcessor;
	/* Priority of this callback */
	unsigned long ulPriority;
	void * pModule;
	struct __libc_ForkCallback * next;
} __LIBC_FORKCALLBACK, *__LIBC_PFORKCALLBACK;

extern int __crtfork_parent1__;
extern int __crtfork_child1__;

#if 1

#define _CRT_FORK_PARENT1(priority, fun1, fun2) __LIBC_FORKCALLBACK __aux_fcp##fun1 = {(__LIBC_FORKCALLBACKFUNCTION) fun1, (__LIBC_FORKCALLBACKFUNCTION) fun2, priority, NULL, NULL }; \
__asm__ (".stabs \"___crtfork_parent1__\", 25, 0, 0, ___aux_fcp" #fun1);

#define _CRT_FORK_CHILD1(priority, fun1) __LIBC_FORKCALLBACK __aux_fcc##fun1 = {(__LIBC_FORKCALLBACKFUNCTION) fun1, NULL, priority, NULL, NULL }; \
__asm__ (".stabs \"___crtfork_child1__\", 25, 0, 0, ___aux_fcc" #fun1);

#else

#define _CRT_FORK_PARENT1(priority, fun1, fun2)  \
__asm__ (".stabs \"___crtfork_parent1__\", 23, 0, 0, _" #fun1); \
__asm__ (".stabs \"___crtfork_parent1__\", 23, 0, 0, _" #fun2); \
__asm__ (".stabs \"___crtfork_parent1__\", 21, 0, 0, " #priority); \
__asm__ (".stabs \"___crtfork_parent1__\", 21, 0, 0, 0"); \
__asm__ (".stabs \"___crtfork_parent1__\", 21, 0, 0, 0");

#define _CRT_FORK_CHILD1(priority, fun1) \
__asm__ (".stabs \"___crtfork_child1__\", 23, 0, 0, _" #fun1); \
__asm__ (".stabs \"___crtfork_child1__\", 21, 0, 0, 0"); \
__asm__ (".stabs \"___crtfork_child1__\", 21, 0, 0, " #priority); \
__asm__ (".stabs \"___crtfork_child1__\", 21, 0, 0, 0"); \
__asm__ (".stabs \"___crtfork_child1__\", 21, 0, 0, 0");

#endif

typedef struct __libc_ForkModule
{
	/** Structure version. (Initially 'FM01' as viewed in hex editor.) */
	unsigned int    iMagic;
	/** Fork callback function */
	int           (*pfnAtFork)(struct __libc_ForkModule *pModule,
				__LIBC_PFORKHANDLE pForkHandle, enum __LIBC_CALLBACKOPERATION enmOperation);
	/** Pointer to the _CRT_FORK_PARENT1 set vector.
	 * It's formatted as __LIBC_FORKCALLBACKPARENT. */
	void  *pvParentVector1;
	/** Pointer to the _CRT_FORK_CHILD1 set vector.
	 * It's formatted as __LIBC_FORKCALLBACKCHILD. */
	void  *pvChildVector1;
	/** Data segment base address. */
	void           *pvDataSegBase;
	/** Data segment end address (exclusive). */
	void           *pvDataSegEnd;
	/** Reserved - must be zero. */
	int             iReserved1;
} __LIBC_FORKMODULE, *__LIBC_PFORKMODULE; /* urg! conventions */

typedef struct __libc_ForkBufferHead
{
	__LIBC_FORKCALLBACKFUNCTION pfnProcessor; /* address of function to process the buffer (generally, the same as caller from parent) */
	__LIBC_PFORKMODULE pModule; /* module of function to process the buffer (generally, the same as caller from parent) */
	int iRc; /* rc of the previous operation */
	unsigned long ulCompletionCallbacksNum;
	void * pMemAddress; /* memory address of the start of memory which part is the data in buffer */
	unsigned long ulMemSize; /* full size of memory which part is the data in buffer */
	unsigned long ulPosition; /* destination position of data in buffer */
	unsigned long ulDataSize; /* size of data in buffer */
} __LIBC_FORKBUFFERHEAD, *__LIBC_PFORKBUFFERHEAD;

 /* The function to be called when fork() is about to complete (the fork() result is established so to
  * A zero rc argument indicates success, a non zero rc argument indicates failure.
  */
typedef int (*__LIBC_COMPLETIONCALLBACK)(__LIBC_PFORKHANDLE pForkHandle, int rc, void *pvArg);

typedef struct __libc_ForkCompletionCallback
{
	__LIBC_COMPLETIONCALLBACK pfnCallback;
	enum __LIBC_FORKSIDE enmContext;
	void * pvArg;
} __LIBC_FORKCOMPLETIONCALLBACK, * __LIBC_PFORKCOMPLETIONCALLBACK;

#define MAX_COMPLETION_CALLBACKS_NUM 10

/* Fork handle for the process */
extern __LIBC_PFORKHANDLE __libc_gpThisForkHandle;

typedef struct __libc_ForkModuleList
{
	struct __libc_ForkModuleList * next;
	__LIBC_PFORKMODULE pModule;
} __LIBC_FORKMODULELIST, *__LIBC_PFORKMODULELIST;

/* callback return codes */
/* no error */
#define __LIBC_FORK_NO_ERROR	0
/* fatal error, non continuable */
#define __LIBC_FORK_ERROR_FATAL	1
/* continuable error */
#define __LIBC_FORK_ERROR_CONT	2

/**
 * Register a forkable module. Called by crt0 and dll0.
 *
 * The call links pModule into the list of forkable modules
 * which is maintained in the process block.
 *
 * @returns 0 on normal process startup.
 * @returns 1 on forked child process startup.
 *          The caller should respond by not calling any _DLL_InitTerm
 *          or similar constructs.
 * @returns negative on failure.
 *          The caller should return from the dll init returning FALSE
 *          or DosExit in case of crt0. _atfork_callback() will take
 *          care of necessary module initiation.
 * @param   pModule     Pointer to the fork module structure for the
 *                      module which is to registered.
 */
int __libc_ForkRegisterModule(__LIBC_FORKMODULE *pModule);

/*
 * Initializes fork handle - open or create shared memory and semaphores
 *
 * @returns 0 on normal initializing.
 * @returns negative on failure.
 * @param   enmSide     either __LIBC_FORKSIDE_PARENT or __LIBC_FORKSIDE_CLIENT
 * @param   ppForkHandle     Pointer to the container where pointer of 
 *                           created fork handle will be returned
 */
int __libc_forkInitHandle(enum __LIBC_FORKSIDE enmSide, __LIBC_PFORKHANDLE * ppForkHandle);

/*
 * DeInitializes fork handle - free fork buffer in shared memory and semaphores
 *
 * @param   pForkHandle Handle of the current fork operation.
 */
void __libc_forkDeInitHandle(__LIBC_PFORKHANDLE pForkHandle);

/*
 * Do fork parent operations
 *
 * @returns	0 on success for child fork, pid on success for parent fork.
 * @returns	negative on failure
 * @param	address of stack on enter to __fork function + 4 + pushed registers
 */
int __libc_forkParent(void * pStackAddress);

/* Detect if we are the forked child by arg parsing
 *
 * @return	0 if we are the fork child
 * @return	1 otherwise
 */
int __libc_ForkChildArgDetect(void);

/* Detect if we are the forked child and thus initiate the fork buffer
 *
 * @return	0 if we are the fork child
 * @return	1 otherwise
 */
int __libc_forkChildDetect(void);

/* Does forking of child process
 *
 * @param	pForkHandle  - fork handler
 */
void __libc_forkChildProcess(__LIBC_PFORKHANDLE pForkHandle);

/**
 * Duplicating a number of pages from pMemStart to pMemEnd taking data from pMemSource.
 * @returns	one of callback return codes.
 * @param	pForkHandle Handle of the current fork operation.
 * @param	pMemStart		Pointer to start of the pages. Rounded down.
 * @param	pMemEnd			Pointer to end of the pages. Rounded up.
 * @param	iFlags		__LIBC_FORK_ONLY_DIRTY means checking whether the
 *						pages are actually dirty before bothering touching
 *						and copying them. (Using the partically broken
 *						DosQueryMemState() API.)
 * 						__LIBC_FORK_ALL means not to bother checking, but
 *						just go ahead copying all the pages.
 */
int __libc_duplicatePages(__LIBC_PFORKHANDLE pForkHandle, void * pMemStart, void * pMemEnd, int iFlags);

/**
 * Duplicating a number of pages from pMemSource to fork memory buffer of ulMemSize size and send it to other side
 * The pfnProcessor, pModule and enmContent fields of fork buffer header are left untouched and should 
 *   be filled prior to calling that function
 * @returns	one of callback return codes.
 * @param	pForkHandle Handle of the current fork operation.
 * @param	pMemSource		Pointer to memory source.
 * @param	ulMemSize		Size of memory.
 * @param	iFlags			Some flags (__LIBC_FORK_ONLY_DIRTY ?).
 *
 */
int __libc_memDupFromSrc(__LIBC_PFORKHANDLE pForkHandle, char * pMemSource, unsigned long ulMemSize, int iFlags);

/**
 * Duplicating a number of pages of ulDataSize size from pMemSource 
 *   to ulDataPosition in pMemDestination (of ulMemSize total size)
 *
 * @returns	one of callback return codes.
 * @param	pForkHandle		Handle of the current fork operation.
 * @param	pMemSource		Pointer to memory source.
 * @param	pMemDestination		Pointer to memory destination.
 * @param	ulMemSize		Size of memory.
 * @param	ulDataPosition		Postition of data from pMemSource in pMemDestination
 * @param	ulDataSize		Size of data from pMemSource in pMemDestination
 * @param	iFlags			Some flags
 *
 */
int __libc_memDupToDest(__LIBC_PFORKHANDLE pForkHandle, char * pMemDestination, char * pMemSource, unsigned long ulMemSize, unsigned long ulDataPosition, unsigned long ulDataSize, int iFlags);

/**
 * Flush the fork() buffer. Meaning taking what ever is in the fork buffer
 * and let the child process it.
 * This might be desired to get the result of a pfnInvoke() in a near
 * synchornous way.
 * @returns	one of callback return codes.
 * @param	pForkHandle Handle of the current fork operation.
 */
int __libc_flush(__LIBC_PFORKHANDLE pForkHandle);

/**
 * Register a fork() completion callback.
 *
 * Use this primitive to do post fork() cleanup.
 * The callbacks are executed first in the child, then in the parent.
 *
 * @returns	0 on success.
 * @returns appropriate non-zero error code on failure. (Usually ENOMEM.)
 * @param   pForkHandle Handle of the current fork operation.
 * @param   pfnCallback Pointer to the function to call back.
 *                      This will be called when fork() is about to
 *                      complete (the fork() result is established so to
 *                      speak). A zero rc argument indicates success,
 *                      a non zero rc argument indicates failure.
 * @param   pvArg       Argument to pass to pfnCallback as 3rd argument.
 * @param   enmContext  __LIBC_FORKCTX_CHILD, __LIBC_FORKCTX_PARENT, or
 *                      __LIBC_FORKCTX_BOTH.
 *                      (mental note: check up the naming convention for enums!)
 * @remark	Use with care, the memory used to remember these is taken from the
 *          fork buffer.
 */
int __libc_registerCompletionCallback(__LIBC_PFORKHANDLE pForkHandle, __LIBC_COMPLETIONCALLBACK pfnCallback, void *pvArg, enum __LIBC_FORKSIDE enmContext);

/**
 * Register a fork() completion callback.
 *
 * Use this primitive to do post fork() cleanup.
 * The callbacks are executed first in the child, then in the parent.
 *
 * @returns	one of callback return codes.
 * @param   pForkHandle Handle of the current fork operation.
 * @param   rc		Fork operation return code.
 *
 */
int __libc_callCompletionCallback(__LIBC_PFORKHANDLE pForkHandle, int iRc);

/**
 * Data segment duplicating callback.
 *
 * @returns	one of callback return codes.
 * @param	pModule		Pointer to fork module structure.
 * @param	pForkHandle	Handle of the current fork operation.
 * @param	enmOperation	Performing operation
 * @param	pForkBuffer	Pointer to fork buffer.
 */
int __libc_DuplicateDataSegmentCallback(__LIBC_PFORKMODULE pModule, __LIBC_PFORKHANDLE pForkHandle, enum __LIBC_CALLBACKOPERATION enmOperation, __LIBC_PFORKBUFFERHEAD pForkBuffer);

/**
 * Memoty allocations from pAllocMemLog duplicating callback.
 *
 * @returns	one of callback return codes.
 * @param	pModule		Pointer to fork module structure.
 * @param	pForkHandle	Handle of the current fork operation.
 * @param	enmOperation	Performing operation
 * @param	pForkBuffer	Pointer to fork buffer.
 */
int __libc_DuplicateMemoryCallback(__LIBC_PFORKMODULE pModule, __LIBC_PFORKHANDLE pForkHandle, enum __LIBC_CALLBACKOPERATION enmOperation, __LIBC_PFORKBUFFERHEAD pForkBuffer);

/**
 * Stack duplicating callback.
 *
 * @returns	one of callback return codes.
 * @param	pModule		Ignored.
 * @param	pForkHandle	Handle of the current fork operation.
 * @param	enmOperation	Performing operation
 * @param	pForkBuffer	Pointer to fork buffer.
 *
 * @remark 	This callback is not to be registered by _CRT_FORK_PARENT1,
 *		but to be called manually at the end of all other callbacks.
 */
int __libc_DuplicateStackCallback(__LIBC_PFORKMODULE pModule, __LIBC_PFORKHANDLE pForkHandle, enum __LIBC_CALLBACKOPERATION enmOperation, __LIBC_PFORKBUFFERHEAD pForkBuffer);

/**
 * LIBC handle duplicating callback.
 *
 * @returns	one of callback return codes.
 * @param	pModule		Ignored.
 * @param	pForkHandle	Ignored.
 * @param	enmOperation	Performing operation
 * @param	pForkBuffer	Ignored.
 *
 * @remark 	To be called from __crtfork_child1__ vector.
 */
int __libc_DuplicateHandlesCallback(__LIBC_PFORKMODULE pModule, __LIBC_PFORKHANDLE pForkHandle, enum __LIBC_CALLBACKOPERATION enmOperation, __LIBC_PFORKBUFFERHEAD pForkBuffer);

/**
 * LIBC rmutex and DOS API semaphores duplicating callback.
 *
 * @returns	one of callback return codes.
 * @param	pModule		Ignored.
 * @param	pForkHandle	Ignored.
 * @param	enmOperation	Performing operation
 * @param	pForkBuffer	Ignored.
 *
 * @remark 	To be called from __crtfork_child1__ vector.
 */
int __libc_DuplicateSemCallback(__LIBC_PFORKMODULE pModule, __LIBC_PFORKHANDLE pForkHandle, enum __LIBC_CALLBACKOPERATION enmOperation, __LIBC_PFORKBUFFERHEAD pForkBuffer);

/**
 * LIBC locale objects duplicating callback.
 *
 * @returns	one of callback return codes.
 * @param	pModule		Ignored.
 * @param	pForkHandle	Ignored.
 * @param	enmOperation	Performing operation
 * @param	pForkBuffer	Ignored.
 *
 * @remark 	To be called from __crtfork_child1__ vector.
 */
int __libc_DuplicateLocaleCallback(__LIBC_PFORKMODULE pModule, __LIBC_PFORKHANDLE pForkHandle, enum __LIBC_CALLBACKOPERATION enmOperation, __LIBC_PFORKBUFFERHEAD pForkBuffer);

/**
 * LIBC _sys_xreg exception handler duplicating callback.
 *
 * @returns	one of callback return codes.
 * @param	pModule		Ignored.
 * @param	pForkHandle	Ignored.
 * @param	enmOperation	Performing operation
 * @param	pForkBuffer	Ignored.
 *
 * @remark 	To be called from __crtfork_child1__ vector with the lowest priority.
 */
int __libc_DuplicateExceptionHandlerCallback(__LIBC_PFORKMODULE pModule, __LIBC_PFORKHANDLE pForkHandle, enum __LIBC_CALLBACKOPERATION enmOperation, __LIBC_PFORKBUFFERHEAD pForkBuffer);

/**
 * LIBC default atfork callback
 *
 * @returns	one of callback return codes.
 * @param	pModule		Ignored.
 * @param	pForkHandle	Ignored.
 * @param	enmOperation	Performing operation
 *
 */
int __libc_AtForkCallback(__LIBC_PFORKMODULE pModule, __LIBC_PFORKHANDLE pForkHandle, enum __LIBC_CALLBACKOPERATION enmOperation);

#pragma pack()

#if defined (__cplusplus)
}
#endif

#endif /* __WITH_FORK */

#endif /* __InnoTekLIBC_fork_h__ */
